var _extends =
  Object.assign ||
  function(target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];
      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }
    return target;
  };

import React, { Component } from 'react';
import {
  View,
  Animated,
  PanResponder,
  Platform,
  Dimensions,
  LayoutAnimation,
  UIManager
} from 'react-native';

const SCREEN_WIDTH = Dimensions.get('window').width;

const SWIPE_THRESHOLD = 0.25 * SCREEN_WIDTH;

class Swiper extends Component {
  constructor(...args) {
    var _temp;

    return (_temp = super(...args)), (this.state = {
      index: 0
    }), (this.panResponder = PanResponder.create({
      onStartShouldSetPanResponder: () => true, // when put finger on the screen
      onPanResponderMove: (event, gesture) => {
        this.position.setValue({ x: gesture.dx, y: gesture.dy });
        if (gesture.dx > 0) {
          this.props.onSwipeRight(gesture.dx);
        }

        if (gesture.dx < 0) {
          this.props.onSwipeLeft(gesture.dx);
        }
        this.props.onSwipe(gesture.dx);
      }, // when moving finger around screen
      onPanResponderRelease: (event, gesture) => {
        if (gesture.dx > SWIPE_THRESHOLD) {
          this.forceSwipe('right');
        } else if (gesture.dx < -SWIPE_THRESHOLD) {
          this.forceSwipe('left');
        } else {
          this.resetPosition();
        }
        if (this.props.onReleaseSwipe) {
          this.props.onReleaseSwipe();
        }
      } // When finger release the screen
    })), (this.position = new Animated.ValueXY()), (this.reset = () =>
      new Promise(resolve => {
        this.setState({ index: 0 }, resolve);
      })), _temp;
  }

  componentWillReceiveProps(nextPorps) {
    if (nextPorps.data !== this.props.data) {
      this.reset();
    }
  }

  componentWillUpdate() {
    if (UIManager.setLayoutAnimationEnabledExperimental) {
      UIManager.setLayoutAnimationEnabledExperimental(true);
    }
    LayoutAnimation.spring();
  }

  onSwipeComplete(direction) {
    const {
      data,
      onCompleteSwipeLeft,
      onCompleteSwipeRight,
      onCompleteSwipe,
      onNoCards
    } = this.props;
    this.position.setValue({ x: 0, y: 0 });
    this.setState(
      ({ index }) => ({ index: index + 1 }),
      () => {
        onCompleteSwipe(this.state.index);
        if (direction === 'right') {
          onCompleteSwipeRight(this.state.index);
        } else onCompleteSwipeLeft(this.state.index);
        if (this.state.index >= data.length && onNoCards) {
          onNoCards();
        }
      }
    );
  }

  forceSwipe(direction = 'right') {
    const x = SCREEN_WIDTH * (direction === 'right' ? 1 : -1);
    Animated.timing(this.position, {
      toValue: { x, y: 0 },
      duration: 250
    }).start(() => this.onSwipeComplete(direction));
  }

  currentCardStyle() {
    // interpolation between dx & rotation degree
    const rotate = this.position.x.interpolate({
      inputRange: [-SCREEN_WIDTH * 1.5, 0, SCREEN_WIDTH * 1.5],
      outputRange: ['-120deg', '0deg', '120deg']
    });
    const style = _extends({}, this.position.getLayout(), {
      transform: [{ rotate }]
    });
    if (Platform.OS !== 'ios') {
      style.elevation = 100;
    }
    return style;
  }

  resetPosition() {
    Animated.spring(this.position, {
      toValue: { x: 0, y: 0 }
    }).start();
  }

  renderCards() {
    return this.props.data
      .map((r, index) => {
        if (index < this.state.index) return null;
        let style = [styles.card];
        let props = { key: JSON.stringify(r) };
        if (index === this.state.index) {
          style = [...style, this.currentCardStyle()];
          props = _extends({}, props, this.panResponder.panHandlers);
        } else {
          style = [...style, { top: (index - this.state.index) * 10 }];
        }
        props = _extends({}, props, { style });
        return React.createElement(
          Animated.View,
          props,
          this.props.renderCard(r, index)
        );
      })
      .reverse();
  }

  render() {
    return React.createElement(
      View,
      null,
      this.state.index >= this.props.data.length ? null : this.renderCards()
    );
  }
}

Swiper.defaultProps = {
  onCompleteSwipe: () => {},
  onCompleteSwipeRight: () => {},
  onCompleteSwipeLeft: () => {},
  onSwipe: () => {},
  onSwipeRight: () => {},
  onSwipeLeft: () => {}
  //  onReleaseSwipe:
};
const styles = {
  card: {
    position: 'absolute',
    width: SCREEN_WIDTH
  }
};
export default Swiper;
